﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ThexConvertGUI
{
    public partial class Form1 : Form
    {
        string FolderPath = "";
        string selectedFile = "";
        string fullPath2File = "";
        uint[,] frame = new uint[384, 288];
        Bitmap bmp = new Bitmap(384, 288);
        ushort[,] palleteArr = new ushort[1001, 3];
        int gmaxValue = 0;
        int gminValue = 0;

        public Form1()
        {
            InitializeComponent();
            loadPaletteFiles();
            loadLocationPath();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog folderBrowserDialog1 = new FolderBrowserDialog();
            folderBrowserDialog1.RootFolder = Environment.SpecialFolder.Desktop;
            if (textBox1.Text.Length > 0) folderBrowserDialog1.SelectedPath = textBox1.Text;

            DialogResult result = folderBrowserDialog1.ShowDialog();

            if (result == DialogResult.OK)
            {
                FolderPath = folderBrowserDialog1.SelectedPath;
                textBox1.Text = FolderPath;
                DirectoryInfo dinfo = new DirectoryInfo(FolderPath);
                FileInfo[] Files = dinfo.GetFiles("*.csv");
                listBox1.Items.Clear();
                foreach (FileInfo file in Files)
                {
                    listBox1.Items.Add(file.Name.ToString());
                }
            }
        }

        private void listBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            selectedFile = listBox1.SelectedItem.ToString();

            fullPath2File = FolderPath + @"\" + selectedFile;

            saveLocationPath(FolderPath);

            readFrameFromCSV();

            displayImageFromFrame(); 
        }

        private void displayImageFromFrame()
        {
            int minValue = 60000;
            int maxValue = 0;
            int maxmin = 0;
            int v = 0;
            uint val = 0;

            for (int y = 0; y < 288; y++)
            {
                for (int x = 0; x < 384; x++)
                {
                    val = frame[x, y];
                    
                    if (val > maxValue)
                    {
                        maxValue = (int)val;
                    }
                    if (val < minValue)
                    {
                        minValue = (int)val;
                    }
                }
            }

            if (!cbFixedRange.Checked)
            {
                gmaxValue = maxValue;
                gminValue = minValue;
            }
            else
            {
                maxValue = gmaxValue;
                minValue = gminValue;
            }

            lblMaxRaw.Text = ((float)maxValue / 100).ToString();
            lblMinRaw.Text = ((float)minValue / 100).ToString();

            //sliders correction:
            minValue = minValue + (trackBar1.Value - 1000);
            maxValue = maxValue - (trackBar2.Value - 1000);

            frame[383, 287] = frame[382, 287];

            for (int y = 0; y < 288; y++)
            {
                for (int x = 0; x < 384; x++)
                {
                    v = (int)frame[x, y];
                    maxmin = maxValue;
                    maxmin -= minValue;
                    if (maxmin == 0)
                        maxmin = 1;

                    v = (v - (int)minValue) * 999 / (int)maxmin;
                    if (v < 0)
                        v = 0;
                    if (v > 999)
                        v = 999;
                    bmp.SetPixel(x, y, Color.FromArgb(palleteArr[v, 0], palleteArr[v, 1], palleteArr[v, 2]));
                    //bmp.SetPixel(x, y, paletteImg.GetPixel((int)v, 0));
                }
            }
            pictureBox1.Image = bmp;
        }

        private void trackBar1_ValueChanged(object sender, EventArgs e)
        {
            displayImageFromFrame();
        }

        private void trackBar2_ValueChanged(object sender, EventArgs e)
        {
            displayImageFromFrame();
        }
        private void saveLocationPath(string FolderPath)
        {
            if (File.Exists("savedloc.txt"))
            {
                File.Delete("savedloc.txt");
            }
            //save path to file:
            using (StreamWriter newTask = new StreamWriter("savedloc.txt", false))
            {
                newTask.WriteLine(FolderPath);
            }
        }
        private void loadLocationPath()
        {
            if (File.Exists("savedloc.txt"))
            {
                FolderPath = File.ReadAllLines("savedloc.txt").First();
                if (Directory.Exists(FolderPath)) {
                    textBox1.Text = FolderPath;
                    DirectoryInfo dinfo = new DirectoryInfo(FolderPath);
                    FileInfo[] Files = dinfo.GetFiles("*.csv");
                    foreach (FileInfo file in Files)
                    {
                        listBox1.Items.Add(file.Name.ToString());
                    }
                }
            }
        }
        private void readFrameFromCSV()
        {
            int ln = 0;
            uint val = 0;

            Array.Clear(frame, 0, frame.Length);
            string[] vrstica;
            string lineNew = "";
            foreach (string line in File.ReadLines(fullPath2File, Encoding.Default))
            {
                if (ln < 289)
                {
                    lineNew = line.Replace("\"", "").Replace(",", "").Replace(".", "");
                    vrstica = lineNew.Split('\t');
                    for (int i = 0; i < vrstica.Length - 1; i++)
                    {
                        val = uint.Parse(vrstica[i]);
                        frame[i, ln] = val;
                    }
                }
                ln++;
            }
        }

        private void saveImage()
        {
            FileInfo fileDest = new FileInfo(FolderPath + @"\export\gui_" + selectedFile.Replace(".csv", "") + ".png");
            fileDest.Directory.Create();
            bmp.Save(fileDest.FullName);
        }

        private void button2_Click(object sender, EventArgs e)
        {
            saveImage();
        }

        private void cbPal_SelectedIndexChanged(object sender, EventArgs e)
        {
            ComboItem newPal = (ComboItem)cbPal.SelectedItem;
            Bitmap paletteImg = new Bitmap(newPal.Key);
            Color picColor;

            for (int i = 0; i < 1001; i++)
            {
                picColor = paletteImg.GetPixel(i, 0);
                palleteArr[i, 0] = picColor.R;
                palleteArr[i, 1] = picColor.G;
                palleteArr[i, 2] = picColor.B;
            }

            paletteImg.RotateFlip(RotateFlipType.Rotate270FlipNone);
            pictureBox4.Image = paletteImg;

            //redraw image:
            displayImageFromFrame();
        }

        class ComboItem
        {
            public string Key { get; set; }
            public string Value { get; set; }
            public ComboItem(string key, string value)
            {
                Key = key; Value = value;
            }
            public override string ToString()
            {
                return Value;
            }
        }

        private void loadPaletteFiles() {
            var pngFiles = new DirectoryInfo(Directory.GetCurrentDirectory() + "\\palette").GetFiles("*.png");

            foreach (FileInfo file in pngFiles)
            {
                cbPal.Items.Add(new ComboItem(file.FullName, file.Name.Replace(".png", "")));
            }
            cbPal.SelectedIndex = 1;
        }

        private void button3_Click(object sender, EventArgs e)
        {
            string origName = "";
            string newName = "";
            string fileFolder = textBox1.Text;

            for (int i=0; i<listBox1.Items.Count; i++)
            {
                origName = listBox1.Items[i].ToString();
                if (origName.Substring(0, 4) == "IMG-")
                {
                    newName = origName.Substring(24);
                    File.Move(fileFolder + @"\" + origName, fileFolder + @"\" + newName);
                }
            }

            DirectoryInfo dinfo = new DirectoryInfo(fileFolder);
            FileInfo[] Files = dinfo.GetFiles("*.csv");
            listBox1.Items.Clear();
            foreach (FileInfo file in Files)
            {
                listBox1.Items.Add(file.Name.ToString());
            }
        }
    }
}